package javax.swing;

import java.util.Enumeration;
import java.awt.event.KeyEvent;
import java.applet.Applet;
import java.awt.Window;
import sun.awt.EmbeddedFrame;
import java.util.Vector;
import java.awt.Container;
import java.util.Hashtable;

class KeyboardManager
{
    static KeyboardManager currentManager;
    Hashtable<Container, Hashtable> containerMap;
    Hashtable<KeyboardManager.ComponentKeyStrokePair, Container> componentKeyStrokeMap;
    
    KeyboardManager() {
        this.containerMap = new Hashtable();
        this.componentKeyStrokeMap = new Hashtable();
    }
    
    public static KeyboardManager getCurrentManager() {
        return KeyboardManager.currentManager;
    }
    
    public static void setCurrentManager(KeyboardManager currentManager) {
        KeyboardManager.currentManager = currentManager;
    }
    
    public void registerKeyStroke(KeyStroke key, JComponent component) {
        Container topAncestor = getTopAncestor(component);
        if (topAncestor == null) {
            return;
        }
        Hashtable<KeyStroke, JComponent> registerNewTopContainer = (Hashtable)this.containerMap.get(topAncestor);
        if (registerNewTopContainer == null) {
            registerNewTopContainer = this.registerNewTopContainer(topAncestor);
        }
        Object value = registerNewTopContainer.get(key);
        if (value == null) {
            registerNewTopContainer.put(key, component);
        }
        else if ((value instanceof Vector)) {
            Vector<JComponent> vector = (Vector)value;
            if (!vector.contains(component)) {
                vector.addElement(component);
            }
        }
        else if ((value instanceof JComponent)) {
            if (value != component) {
                Vector value2 = new Vector();
                value2.addElement((JComponent)value);
                value2.addElement(component);
                registerNewTopContainer.put(key, value2);
            }
        }
        else {
            System.out.println("Unexpected condition in registerKeyStroke");
            Thread.dumpStack();
        }
        this.componentKeyStrokeMap.put(new KeyboardManager.ComponentKeyStrokePair(component, key), topAncestor);
        if ((topAncestor instanceof EmbeddedFrame)) {
            ((EmbeddedFrame)topAncestor).registerAccelerator(key);
        }
        return;
    }
    
    private static Container getTopAncestor(JComponent component) {
        Container container = component.getParent();
        while (container != null) {
            if ((container instanceof Window && ((Window)container).isFocusableWindow()) || container instanceof Applet || container instanceof JInternalFrame) {
                return container;
            }
            container = container.getParent();
        }
        return null;
    }
    
    public void unregisterKeyStroke(KeyStroke key, JComponent obj) {
        KeyboardManager.ComponentKeyStrokePair componentKeyStrokePair = new KeyboardManager.ComponentKeyStrokePair(obj, key);
        Container container = (Container)this.componentKeyStrokeMap.get(componentKeyStrokePair);
        if (container == null) {
            return;
        }
        Hashtable<Object, Object> hashtable = (Hashtable)this.containerMap.get(container);
        if (hashtable == null) {
            Thread.dumpStack();
            return;
        }
        Object value = hashtable.get(key);
        if (value == null) {
            Thread.dumpStack();
            return;
        }
        if (value instanceof JComponent && value == obj) {
            hashtable.remove(key);
        }
        else if ((value instanceof Vector)) {
            Vector<Object> vector = (Vector)value;
            vector.removeElement(obj);
            if (vector.isEmpty()) {
                hashtable.remove(key);
            }
        }
        if (hashtable.isEmpty()) {
            this.containerMap.remove(container);
        }
        this.componentKeyStrokeMap.remove(componentKeyStrokePair);
        if ((container instanceof EmbeddedFrame)) {
            ((EmbeddedFrame)container).unregisterAccelerator(key);
        }
        return;
    }
    
    public boolean fireKeyboardAction(KeyEvent keyEvent, boolean b, Container key) {
        if (keyEvent.isConsumed()) {
            System.out.println("Acquired pre-used event!");
            Thread.dumpStack();
        }
        KeyStroke keyStroke = null;
        KeyStroke keyStroke2;
        if (keyEvent.getID() == 400) {
            keyStroke2 = KeyStroke.getKeyStroke(keyEvent.getKeyChar());
        }
        else {
            if (keyEvent.getKeyCode() != keyEvent.getExtendedKeyCode()) {
                keyStroke = KeyStroke.getKeyStroke(keyEvent.getExtendedKeyCode(), keyEvent.getModifiers(), !b);
            }
            keyStroke2 = KeyStroke.getKeyStroke(keyEvent.getKeyCode(), keyEvent.getModifiers(), !b);
        }
        Hashtable<Object, Object> hashtable = (Hashtable)this.containerMap.get(key);
        if (hashtable != null) {
            Object obj = null;
            if (keyStroke != null) {
                obj = hashtable.get(keyStroke);
                if (obj != null) {
                    keyStroke2 = keyStroke;
                }
            }
            if (obj == null) {
                obj = hashtable.get(keyStroke2);
            }
            if (obj != null) {
                if ((obj instanceof JComponent)) {
                    JComponent component2 = (JComponent)obj;
                    if (component2.isShowing() && component2.isEnabled()) {
                        this.fireBinding(component2, keyStroke2, keyEvent, b);
                    }
                }
                else if ((obj instanceof Vector)) {
                    Vector<Object> vector = (Vector)obj;
                    for (int i = vector.size() - 1; i >= 0; --i) {
                        JComponent component = (JComponent)vector.elementAt(i);
                        if (component.isShowing() && component.isEnabled()) {
                            this.fireBinding(component, keyStroke2, keyEvent, b);
                            if (keyEvent.isConsumed()) {
                                return true;
                            }
                        }
                    }
                }
                else {
                    System.out.println("Unexpected condition in fireKeyboardAction " + obj);
                    Thread.dumpStack();
                }
            }
        }
        if (keyEvent.isConsumed()) {
            return true;
        }
        if (hashtable != null) {
            Vector<Object> vector2 = (Vector)hashtable.get(JMenuBar.class);
            if (vector2 != null) {
                Enumeration<Object> elements = vector2.elements();
                while (elements.hasMoreElements()) {
                    JMenuBar menuBar = (JMenuBar)elements.nextElement();
                    if (menuBar.isShowing() && menuBar.isEnabled()) {
                        int n = keyStroke != null && !keyStroke.equals(keyStroke2);
                        if (!(n == 0)) {
                            this.fireBinding(menuBar, keyStroke, keyEvent, b);
                        }
                        if (n == 0 || !keyEvent.isConsumed()) {
                            this.fireBinding(menuBar, keyStroke2, keyEvent, b);
                        }
                        if (keyEvent.isConsumed()) {
                            return true;
                        }
                        continue;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        return keyEvent.isConsumed();
    }
    
    void fireBinding(JComponent component, KeyStroke keyStroke, KeyEvent keyEvent, boolean b) {
        if (component.processKeyBinding(keyStroke, keyEvent, 2, b)) {
            keyEvent.consume();
        }
    }
    
    public void registerMenuBar(JMenuBar menuBar) {
        Container topAncestor = getTopAncestor(menuBar);
        if (topAncestor == null) {
            return;
        }
        Hashtable<Object, Vector> registerNewTopContainer = (Hashtable)this.containerMap.get(topAncestor);
        if (registerNewTopContainer == null) {
            registerNewTopContainer = this.registerNewTopContainer(topAncestor);
        }
        Vector<JMenuBar> value = (Object)registerNewTopContainer.get(JMenuBar.class);
        if (value == null) {
            value = new Vector();
            registerNewTopContainer.put(JMenuBar.class, value);
        }
        if (!value.contains(menuBar)) {
            value.addElement(menuBar);
        }
        return;
    }
    
    public void unregisterMenuBar(JMenuBar obj) {
        Container topAncestor = getTopAncestor(obj);
        if (topAncestor == null) {
            return;
        }
        Hashtable<Object, Object> hashtable = (Hashtable)this.containerMap.get(topAncestor);
        if (hashtable != null) {
            Vector<Object> vector = (Vector)hashtable.get(JMenuBar.class);
            if (vector != null) {
                vector.removeElement(obj);
                if (vector.isEmpty()) {
                    hashtable.remove(JMenuBar.class);
                    if (hashtable.isEmpty()) {
                        this.containerMap.remove(topAncestor);
                    }
                }
            }
        }
        return;
    }
    
    protected Hashtable registerNewTopContainer(Container key) {
        Hashtable<Object, Object> value = new Hashtable();
        this.containerMap.put(key, value);
        return value;
    }
    
    static {
        KeyboardManager.currentManager = new KeyboardManager();
    }
    
    class ComponentKeyStrokePair
    {
        Object component;
        Object keyStroke;
        
        public ComponentKeyStrokePair(Object component, Object keyStroke) {
            this.component = component;
            this.keyStroke = keyStroke;
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof KeyboardManager.ComponentKeyStrokePair)) {
                return false;
            }
            KeyboardManager.ComponentKeyStrokePair componentKeyStrokePair = (KeyboardManager.ComponentKeyStrokePair)o;
            return this.component.equals(componentKeyStrokePair.component) && this.keyStroke.equals(componentKeyStrokePair.keyStroke);
        }
        
        @Override
        public int hashCode() {
            return this.component.hashCode() * this.keyStroke.hashCode();
        }
    }
}
